@@ -523,8 +523,8 @@ int len;
 			written = write(soc, ptr, left);
 			if (written <= 0) {
 				perror("write");
-				sprintf(logstr, "Write Error : %d\n",errno);
-				pr_log(VLOG_GLOB, LOG_ALL, logstr, ON);
+				pr_log2(VLOG_GLOB, LOG_ALL, ON,
+						"Write Error : %d\n",errno);
 				return written;
 			}
 			left -= written;
