@@ -4,7 +4,7 @@
 /*                                                              */
 /* IRRd Interface                                               */
 /****************************************************************/
-/* irrdif.c,v 1.9 2007/03/14 02:58:28 kuniaki Exp */
+/* irrdif.c,v 1.10 2008/06/26 05:24:08 kuniaki Exp */
 
 /*
  * Copyright (c) 2006 Atelier Mahoroba
@@ -191,44 +191,87 @@ int irr_search_route(struct in_addr *in, u_short plen)
 
 int sread(char *buf, int len)
 {
-        int     cnt;
-        int     rel;
+        int     	cnt;
+        int    		rel;
+	struct timeval	tval;
+	fd_set		irrset;
 
+	tval.tv_sec  = IRR_SEARCH_TIMEOUT;
+	tval.tv_usec = 0;
         cnt = 0;
 	if (irrsock == 0) return(0);
         bzero(buf, len);
         while(cnt < len) {
-                rel = read(irrsock, (char *)(buf + cnt), (len-cnt));
-                cnt += rel;
+		FD_ZERO(&irrset);
+		FD_SET(irrsock, &irrset);
+		if (select(FD_SETSIZE, NULL, &irrset, NULL, &tval) < 0) {
+			if (errno == EINTR) {
+				printf("Select Return Code EINTR on irrdata\n");
+				continue;
+			}
+			perror("select on irrdata");
+			pr_log2(VLOG_GLOB, LOG_ALL, ON,
+				"Select Error on IRR Check\n");
+			return(0);
+		}
+		if (FD_ISSET(irrsock, &irrset)) {
+                	rel = read(irrsock, (char *)(buf + cnt), (len-cnt));
+                	cnt += rel;
 #ifdef DEBUG
-                printf("DEBUG: Reading %d/%d/%d\n",rel,cnt,len);
-                fflush(stdout);
+                	printf("DEBUG: Reading %d/%d/%d\n",rel,cnt,len);
+                	fflush(stdout);
 #endif
+		} else {
+			/* Data Read Receive Timeout */
+			return(0);
+		}
         }
         *(buf+len) = '\0';
 #ifdef DEBUG
         printf("%d:%s", strlen(buf), buf);
 #endif
+	return(cnt);
 }
 
 int getcontrol(char *type)
 {
-        char    line[STRBUFLEN];
-        char    c;
-        int     cnt;
-        int     ret;
+        char    	line[STRBUFLEN];
+        char    	c;
+        int     	cnt;
+        int     	ret;
+	struct timeval	tval;
+	fd_set		irrset;
 
+	tval.tv_sec  = IRR_SEARCH_TIMEOUT;
+	tval.tv_usec = 0;
 	if (irrsock == 0) return(0);
         bzero(line, STRBUFLEN);
         cnt = 0;
         while(1) {
-                read(irrsock, &c, 1);
-                if (c == '\n') break;
-                if (c == '\r') break;
-                if (cnt == -1) break;
-                if (cnt >= STRBUFLEN) continue;
-                line[cnt] = c;
-                cnt++;
+		FD_ZERO(&irrset);
+		FD_SET(irrsock, &irrset);
+		if (select(FD_SETSIZE, NULL, &irrset, NULL, &tval) < 0) {
+			if (errno == EINTR) {
+				printf("Select Return Code EINTR on irr\n");
+				continue;
+			}
+			perror("select on irr");
+			pr_log2(VLOG_GLOB, LOG_ALL, ON, 
+				"Select Error on IRR Check\n");
+			return(0);
+		}
+		if (FD_ISSET(irrsock, &irrset)) {
+	                read(irrsock, &c, 1);
+                	if (c == '\n') break;
+                	if (c == '\r') break;
+                	if (cnt == -1) break;
+                	if (cnt >= STRBUFLEN) continue;
+                	line[cnt] = c;
+                	cnt++;
+		} else {
+			/* Control Receive Timeout */
+			return(0);
+		}
         }
 
         ret = 0;
@@ -288,8 +331,12 @@ char *readdata()
 #ifdef DEBUG
                         printf("MALLOCED ADDRESS:0x%08X\n", data);
 #endif
-                        sread(data, result);
-                        getdata = 1;
+                        if (sread(data, result) < result) {
+				/* Receive In-Complete */
+				getdata = 0;
+			} else {
+                        	getdata = 1;
+			}
                         break;
 
                 case 'B':
